#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>

#if defined (__FreeBSD__)
#include <sys/telephony.h>
#include <sys/ixjuser.h>
#else
#include <linux/telephony.h>
#include <linux/ixjuser.h>
#endif


static inline int ast_strlen_zero(const char *s)
{
        return (*s == '\0');
}

void
ast_shrink_phone_number(char *n)
{
        int x,y=0;
        for (x=0;n[x];x++)
                if (!strchr("( )-.", n[x]))
                        n[y++] = n[x];
        n[y] = '\0';
}

int ast_isphonenumber(char *n)
{
        int x;
        if (!n || ast_strlen_zero(n))
                return 0;
        for (x=0;n[x];x++)
                if (!strchr("0123456789*#+", n[x]))
                        return 0;
        return 1;
}

int ast_callerid_parse(char *instr, char **name, char **location)
{
        char *ns, *ne;
        char *ls, *le;
        char tmp[256];
        /* Try for "name" <location> format or 
           name <location> format */
        if ((ls = strchr(instr, '<')) && (le = strchr(ls, '>'))) {
                /* Found the location */
                *le = '\0';
                *ls = '\0';
                *location = ls + 1;
                if ((ns = strchr(instr, '\"')) && (ne = strchr(ns + 1, '\"'))) {
                        /* Get name out of quotes */
                        *ns = '\0';
                        *ne = '\0'; 
                        *name = ns + 1;
                        return 0;
                } else {
                        /* Just trim off any trailing spaces */
                        *name = instr;
                        while(!ast_strlen_zero(instr) && (instr[strlen(instr) - 1] < 33))
                                instr[strlen(instr) - 1] = '\0';
                        /* And leading spaces */
                        while(**name && (**name < 33))
                                (*name)++;
                        return 0;
                }
        } else {
                strncpy(tmp, instr, sizeof(tmp)-1);
                ast_shrink_phone_number(tmp);
                if (ast_isphonenumber(tmp)) {
                        /* Assume it's just a location */
                        *name = NULL;
                        *location = instr;
                } else {
                        /* Assume it's just a name.  Make sure it's not quoted though */
                        *name = instr;
                        while(*(*name) && ((*(*name) < 33) || (*(*name) == '\"'))) (*name)++;
                        ne = *name + strlen(*name) - 1;
                        while((ne > *name) && ((*ne < 33) || (*ne == '\"'))) { *ne = '\0'; ne--; }
                        *location = NULL;
                }
                return 0;
        }
        return -1;
}


#define DEFAULT_CALLER_ID	"Default Caller"

int main(int argc, char *argv[])
{
	char *pname = "/dev/phone0";
	int ixj;
	int dsp,ver;
	time_t t;
	struct tm tm;
	char ast_callerid[80] ="";

	PHONE_CID cid;

	printf("Opening %s\n", pname);
	fflush(stdout);

	if ( (ixj = open(pname, O_RDWR))<0){
		printf("error opening %s (%s)\n", pname, strerror(errno));
		exit(-1);
	}

	dsp = ioctl(ixj, IXJCTL_DSP_TYPE);
	ver = ioctl(ixj, IXJCTL_DSP_VERSION);
	printf("DSP type 0x%x\nDSP Version 0x%x\n", dsp, ver);

	ioctl(ixj, IXJCTL_PORT, PORT_POTS);

	if(argc > 1){
		strcpy(ast_callerid,argv[1]);
		printf("Caller Id:   %s\n",ast_callerid);
	}

	char *s;
	time(&t);
	localtime_r(&t,&tm);

	if(&tm != NULL) {
		sprintf(cid.month, "%02d",(tm.tm_mon + 1));
		sprintf(cid.day,   "%02d", tm.tm_mday);
		sprintf(cid.hour,  "%02d", tm.tm_hour);
		sprintf(cid.min,   "%02d", tm.tm_min);
	}

	if(ast_strlen_zero(ast_callerid)){
		strcpy(cid.name,DEFAULT_CALLER_ID);
		cid.number[0]='\0';
	} else {
		char *n, *l;
		char callerid[256] = "";
		strncpy(callerid, ast_callerid, sizeof(callerid) - 1);
		ast_callerid_parse(callerid, &n, &l);
		if (l) {
			ast_shrink_phone_number(l);
		if (!ast_isphonenumber(l))
			l = NULL;
		}
		if (l) strncpy(cid.number, l, sizeof(cid.number));
		if (n) strncpy(cid.name, n, sizeof(cid.name));
	}

#ifdef __linux__
	ioctl(ixj, PHONE_RING_START, &cid);
#else
#ifdef DRV_VER > 100
	ioctl(ixj, PHONE_RING_START, cid);
#else
	ioctl(ixj, PHONE_RING_START, &cid);
#endif
#endif
	printf("Press <enter> to stop ringing\n");
	getchar();
	ioctl(ixj, PHONE_RING_STOP);
	close(ixj);
}
